//------------------------------------------------------------------------------
// multigenHD_horz.v
//
// HD digital video YCbCr 4:2:2 video pattern generator horizontal section
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2004 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Mar 2, 2004
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Mar  2, 2004       XAPP682 version 1.0 release
//
// Other modules instanced in this design:
//
//          None.
//------------------------------------------------------------------------------
/*
This module contains the horizontal sequencer for the HD video pattern 
generator. A block RAM is used as a finite state machine to sequence through
the various horizontal regions of the video patterns. The module outputs a
horizontal region code indicating which horizontal region of the video pattern
is currently active.
*/

module multigenHD_horz (
    input   wire        clk,            // word-rate clock
    input   wire        rst,            // async reset
    input   wire        ce,             // clock enable
    input   wire [2:0]  std,            // selects video format
    input   wire [1:0]  pattern,        // 00 = RP 219 colorbars, X1 = RP 198 checkfield, 10 = 75% colorbars
    input   wire [1:0]  user_opt,       // selects option for the *2 & *3 blocks of RP 219
    input   wire        first_line,     // asserted during first active video line
    input   wire        f,              // odd/even field indicator
    output  wire        v_inc,          // increment vertical counter
    output  wire        trs,            // asserted at start of TRS
    output  wire        xyz,            // asserted during XYZ word of TRS
    output  wire        h,              // horizontal blanking interval indicator
    output  reg  [4:0]  h_region,       // horizontal region code
    output  wire        h_counter_lsb   // LSB of horizontal counter
);

//-----------------------------------------------------------------------------
// Parameter definitions
//

//
// This group of parameters defines the bit widths of various fields in the
// module. Note that if the VID_WIDTH parameter is changed, the video component
// values for the various colors will need to be modified accordingly.
//
parameter VID_WIDTH     = 10;                   // Width of video components
parameter HCNT_WIDTH    = 12;                   // Width of h_counter
parameter HRGN_WIDTH    = 5;                    // Width of h_region counter

parameter H_EVNT_WIDTH  = HCNT_WIDTH - 1;       // Width of h_next_evnt
 
parameter VID_MSB       = VID_WIDTH - 1;        // MS bit # of video data path
parameter HCNT_MSB      = HCNT_WIDTH - 1;       // MS bit # of h_counter
parameter HRGN_MSB      = HRGN_WIDTH - 1;       // MS bit # of h_region counter
parameter H_EVNT_MSB    = H_EVNT_WIDTH - 1;     // MS bit # of h_next_evnt

//
// This group of parameters defines the horizontal regions from the horizontal
// ROM. Note that the three HRGN_USROPTx regions are never generated by the
// horizontal ROM. An encoder will change HRGN_BAR1 to any of these three values
// depending on the user option inputs before feeding the modified horizontal
// region code to the color ROM. The encoder will also determine whether to
// output the HRGN_CEQ_POL0 or POL1 code depending on whether the current frame
// is odd or even.
//
parameter [HRGN_MSB:0]
    HRGN_BAR0           = 0,    // 40% gray
    HRGN_BAR1           = 1,    // 75% white (left part 0% black)
    HRGN_BAR2           = 2,    // left part 75% yellow (right part 0% black)
    HRGN_BAR3           = 3,    // right part 75% yellow (right part 100% white)
    HRGN_BAR4           = 4,    // 75% cyan (middle part 100% white)
    HRGN_BAR5           = 5,    // left part 75% green (left part 100% white)
    HRGN_BAR6           = 6,    // right part 75% green (left part 0% black)
    HRGN_BAR7           = 7,    // left part 75% magenta (right part 0% black) 
    HRGN_BAR8           = 8,    // middle part 75% magenta (-2% black)
    HRGN_BAR9           = 9,    // right part 75% magenta (0% black)
    HRGN_BAR10          = 10,   // left part 75% red (+2% black)
    HRGN_BAR11          = 11,   // middle part 75% red (0% black)
    HRGN_BAR12          = 12,   // right part 75% red (+4% black)
    HRGN_BAR13          = 13,   // 75% blue (0% black)
    HRGN_BAR14          = 14,   // 40% gray
    HRGN_INCV           = 15,   // increment vertical line number
    HRGN_EAV1           = 16,   // first two words of EAV
    HRGN_EAV2_F0        = 17,   // second two words of EAV for field 0
    HRGN_HBLANK         = 18,   // horizontal blanking period
    HRGN_SAV1           = 19,   // first two words of SAV
    HRGN_SAV2_F0        = 20,   // second two words of SAV for field 0
    HRGN_LN             = 21,   // two words of line number
    HRGN_CRC            = 22,   // two words of CRC
    HRGN_SAV2_F1        = 23,   // second two words of SAV for field 1
    HRGN_USROPT1        = 24,   // BAR1 but with 100% white in pat 2, 0% black in pat 3
    HRGN_USROPT2        = 25,   // BAR1 but with +I in pat 2, 0% black in pat 3
    HRGN_USROPT3        = 26,   // BAR1 but with -I in pat 2, +Q in pat 3
    HRGN_EAV2_F1        = 27,   // second two words of EAV for field 1
    HRGN_CEQ_PLL        = 28,   // RP 198 cable equalization pattern
    HRGN_CEQ_POL_0      = 29,   // RP 198 cable eq pattern, polarity word even frame
    HRGN_CEQ_POL_1      = 30,   // RP 198 cable eq pattern, polarity word odd frame
    HRGN_RST            = 31;   // initial state after reset


//-----------------------------------------------------------------------------
// Signal definitions
//

wire    [31:0]          hrom_out;       // HROM output
reg     [HCNT_MSB:0]    h_counter;      // horizontal counter
wire    [H_EVNT_MSB:0]  h_next_evnt;    // next horizontal event
wire                    h_evnt_match;   // output of horizontal event comparator
wire                    hrom_en;        // EN input to horizontal ROM
wire    [HRGN_MSB:0]    h_region_rom;   // current horizontal region
wire    [HRGN_MSB:0]    h_next_region;  // next horizontal region
wire                    h_clr;          // clears the horizontal counter
wire                    usropt_rgn;     // horz region where h_region is affected by user_opt inputs
wire                    ceqpol_rgn;     // horz region where h_region is affected by ceq polarity
wire                    eav2_rgn;       // last two words of EAV
wire                    sav2_rgn;       // last two words of SAV
wire    [31:0]          GND32 = 32'b0000_0000_0000_0000_0000_0000_0000_0000;
wire    [3:0]           GND4 =  4'b0000;


//
// Horizontal ROM
//

// Simulation initialization code HROM
// Created by multigenHD_romgen.v
// Video format mapping:
//   000 =  SMPTE 295M - 1080i  25Hz (1250 lines/frame)
//   001 =  SMPTE 274M - 1080sF 24Hz & 23.98Hz         
//   002 =  SMPTE 274M - 1080i  30Hz & 29.97 Hz        
//   003 =  SMPTE 274M - 1080i  25Hz                   
//   004 =  SMPTE 274M - 1080p  30Hz & 29.97Hz         
//   005 =  SMPTE 274M - 1080p  25Hz                   
//   006 =  SMPTE 274M - 1080p  24Hz & 23.98Hz         
//   007 =  SMPTE 296M - 720p   60Hz & 59.94Hz         
//synthesis translate_off
defparam
HROM.INIT = 36'h00060FFFF,
HROM.SRVAL = 36'h00060FFFF,
HROM.WRITE_MODE = "READ_FIRST",
HROM.INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INITP_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INITP_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INITP_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INITP_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
HROM.INIT_00 = 256'h000746880006424700053BE600043585000328A40002222302011BC200000EE1,
HROM.INIT_01 = 256'h004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9,
HROM.INIT_02 = 256'h00170010011678720115785611B4946001939454011294330991783501907811,
HROM.INIT_03 = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_04 = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_05 = 256'h004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_06 = 256'h00170010011678720115785611B4947D01939454011294330991783501907811,
HROM.INIT_07 = 256'h015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010,
HROM.INIT_08 = 256'h000746880006424700053BE600043585000328A40002222302011BC200000EE1,
HROM.INIT_09 = 256'h004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9,
HROM.INIT_0A = 256'h00170010011678720115785611B4ABC00193ABB40112AB930991783501907811,
HROM.INIT_0B = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_0C = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_0D = 256'h004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_0E = 256'h00170010011678720115785611B4ABDD0193ABB40112AB930991783501907811,
HROM.INIT_0F = 256'h015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010,
HROM.INIT_10 = 256'h000746880006424700053BE600043585000328A40002222302011BC200000EE1,
HROM.INIT_11 = 256'h004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9,
HROM.INIT_12 = 256'h00170010011678720115785611B4896001938954011289330991783501907811,
HROM.INIT_13 = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_14 = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_15 = 256'h004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_16 = 256'h00170010011678720115785611B4897D01938954011289330991783501907811,
HROM.INIT_17 = 256'h015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010,
HROM.INIT_18 = 256'h000746880006424700053BE600043585000328A40002222302011BC200000EE1,
HROM.INIT_19 = 256'h004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9,
HROM.INIT_1A = 256'h00170010011678720115785611B4A4E00193A4D40112A4B30991783501907811,
HROM.INIT_1B = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_1C = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_1D = 256'h004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_1E = 256'h00170010011678720115785611B4A4FD0193A4D40112A4B30991783501907811,
HROM.INIT_1F = 256'h015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010,
HROM.INIT_20 = 256'h000746880006424700053BE600043585000328A40002222302011BC200000EE1,
HROM.INIT_21 = 256'h004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9,
HROM.INIT_22 = 256'h00170010011678720115785611B4896001938954011289330991783501907811,
HROM.INIT_23 = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_24 = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_25 = 256'h004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_26 = 256'h00170010011678720115785611B4897D01938954011289330991783501907811,
HROM.INIT_27 = 256'h015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010,
HROM.INIT_28 = 256'h000746880006424700053BE600043585000328A40002222302011BC200000EE1,
HROM.INIT_29 = 256'h004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9,
HROM.INIT_2A = 256'h00170010011678720115785611B4A4E00193A4D40112A4B30991783501907811,
HROM.INIT_2B = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_2C = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_2D = 256'h004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_2E = 256'h00170010011678720115785611B4A4FD0193A4D40112A4B30991783501907811,
HROM.INIT_2F = 256'h015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010,
HROM.INIT_30 = 256'h000746880006424700053BE600043585000328A40002222302011BC200000EE1,
HROM.INIT_31 = 256'h004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9,
HROM.INIT_32 = 256'h00170010011678720115785611B4ABC00193ABB40112AB930991783501907811,
HROM.INIT_33 = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_34 = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_35 = 256'h004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_36 = 256'h00170010011678720115785611B4ABDD0193ABB40112AB930991783501907811,
HROM.INIT_37 = 256'h015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010,
HROM.INIT_38 = 256'h00072F0800062C27000527E6000423A500031B24000216C3020112A2000009E1,
HROM.INIT_39 = 256'h004F4FF0000E4FCF000D45EE000C3D2D000B3A4C000A378B000934AA000831E9,
HROM.INIT_3A = 256'h00170010011650720115505611B46700019366F4011266D30991503501905011,
HROM.INIT_3B = 256'h015F0012001E0010001D0010001C0010001B0010001A00100019001000180010,
HROM.INIT_3C = 256'h0007001000060010000500100004001000030010000200100001001000000010,
HROM.INIT_3D = 256'h004F4FF0000E0010000D0010000C0010000B0010000A00100009001000080010,
HROM.INIT_3E = 256'h00170010011650720115505611B4671D019366F4011266D30991503501905011,
HROM.INIT_3F = 256'h015F0012001E0010041D003C001C4FCF001B0010001A00100019001000180010;
//synthesis translate_on

RAMB16_S36 HROM (
    .DO     (hrom_out),
    .DOP    (),
    .ADDR   ({std, pattern[0], h_next_region}),
    .CLK    (clk),
    .DI     (GND32),
    .DIP    (GND4),
    .EN     (hrom_en),
    .SSR    (rst),
    .WE     (1'b0)); 

// XST synthesis initialization code HROM
// Created by multigenHD_romgen.v
// Video format mapping:
//   000 =  SMPTE 295M - 1080i  25Hz (1250 lines/frame)
//   001 =  SMPTE 274M - 1080sF 24Hz & 23.98Hz         
//   002 =  SMPTE 274M - 1080i  30Hz & 29.97 Hz        
//   003 =  SMPTE 274M - 1080i  25Hz                   
//   004 =  SMPTE 274M - 1080p  30Hz & 29.97Hz         
//   005 =  SMPTE 274M - 1080p  25Hz                   
//   006 =  SMPTE 274M - 1080p  24Hz & 23.98Hz         
//   007 =  SMPTE 296M - 720p   60Hz & 59.94Hz         
//synthesis attribute INIT of HROM is "00060FFFF"
//synthesis attribute SRVAL of HROM is "00060FFFF"
//synthesis attribute WRITE_MODE of HROM is "READ_FIRST"
//synthesis attribute INITP_00 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_01 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_02 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_03 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_04 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_05 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_06 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_07 of HROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INIT_00 of HROM is "000746880006424700053BE600043585000328A40002222302011BC200000EE1"
//synthesis attribute INIT_01 of HROM is "004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9"
//synthesis attribute INIT_02 of HROM is "00170010011678720115785611B4946001939454011294330991783501907811"
//synthesis attribute INIT_03 of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_04 of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_05 of HROM is "004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_06 of HROM is "00170010011678720115785611B4947D01939454011294330991783501907811"
//synthesis attribute INIT_07 of HROM is "015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010"
//synthesis attribute INIT_08 of HROM is "000746880006424700053BE600043585000328A40002222302011BC200000EE1"
//synthesis attribute INIT_09 of HROM is "004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9"
//synthesis attribute INIT_0A of HROM is "00170010011678720115785611B4ABC00193ABB40112AB930991783501907811"
//synthesis attribute INIT_0B of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_0C of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_0D of HROM is "004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_0E of HROM is "00170010011678720115785611B4ABDD0193ABB40112AB930991783501907811"
//synthesis attribute INIT_0F of HROM is "015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010"
//synthesis attribute INIT_10 of HROM is "000746880006424700053BE600043585000328A40002222302011BC200000EE1"
//synthesis attribute INIT_11 of HROM is "004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9"
//synthesis attribute INIT_12 of HROM is "00170010011678720115785611B4896001938954011289330991783501907811"
//synthesis attribute INIT_13 of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_14 of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_15 of HROM is "004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_16 of HROM is "00170010011678720115785611B4897D01938954011289330991783501907811"
//synthesis attribute INIT_17 of HROM is "015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010"
//synthesis attribute INIT_18 of HROM is "000746880006424700053BE600043585000328A40002222302011BC200000EE1"
//synthesis attribute INIT_19 of HROM is "004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9"
//synthesis attribute INIT_1A of HROM is "00170010011678720115785611B4A4E00193A4D40112A4B30991783501907811"
//synthesis attribute INIT_1B of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_1C of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_1D of HROM is "004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_1E of HROM is "00170010011678720115785611B4A4FD0193A4D40112A4B30991783501907811"
//synthesis attribute INIT_1F of HROM is "015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010"
//synthesis attribute INIT_20 of HROM is "000746880006424700053BE600043585000328A40002222302011BC200000EE1"
//synthesis attribute INIT_21 of HROM is "004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9"
//synthesis attribute INIT_22 of HROM is "00170010011678720115785611B4896001938954011289330991783501907811"
//synthesis attribute INIT_23 of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_24 of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_25 of HROM is "004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_26 of HROM is "00170010011678720115785611B4897D01938954011289330991783501907811"
//synthesis attribute INIT_27 of HROM is "015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010"
//synthesis attribute INIT_28 of HROM is "000746880006424700053BE600043585000328A40002222302011BC200000EE1"
//synthesis attribute INIT_29 of HROM is "004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9"
//synthesis attribute INIT_2A of HROM is "00170010011678720115785611B4A4E00193A4D40112A4B30991783501907811"
//synthesis attribute INIT_2B of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_2C of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_2D of HROM is "004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_2E of HROM is "00170010011678720115785611B4A4FD0193A4D40112A4B30991783501907811"
//synthesis attribute INIT_2F of HROM is "015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010"
//synthesis attribute INIT_30 of HROM is "000746880006424700053BE600043585000328A40002222302011BC200000EE1"
//synthesis attribute INIT_31 of HROM is "004F77F0000E77CF000D68EE000C5C0D000B57CC000A536B00094F2A00084AC9"
//synthesis attribute INIT_32 of HROM is "00170010011678720115785611B4ABC00193ABB40112AB930991783501907811"
//synthesis attribute INIT_33 of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_34 of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_35 of HROM is "004F77F0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_36 of HROM is "00170010011678720115785611B4ABDD0193ABB40112AB930991783501907811"
//synthesis attribute INIT_37 of HROM is "015F0012001E0010041D003C001C77CF001B0010001A00100019001000180010"
//synthesis attribute INIT_38 of HROM is "00072F0800062C27000527E6000423A500031B24000216C3020112A2000009E1"
//synthesis attribute INIT_39 of HROM is "004F4FF0000E4FCF000D45EE000C3D2D000B3A4C000A378B000934AA000831E9"
//synthesis attribute INIT_3A of HROM is "00170010011650720115505611B46700019366F4011266D30991503501905011"
//synthesis attribute INIT_3B of HROM is "015F0012001E0010001D0010001C0010001B0010001A00100019001000180010"
//synthesis attribute INIT_3C of HROM is "0007001000060010000500100004001000030010000200100001001000000010"
//synthesis attribute INIT_3D of HROM is "004F4FF0000E0010000D0010000C0010000B0010000A00100009001000080010"
//synthesis attribute INIT_3E of HROM is "00170010011650720115505611B4671D019366F4011266D30991503501905011"
//synthesis attribute INIT_3F of HROM is "015F0012001E0010041D003C001C4FCF001B0010001A00100019001000180010"


assign h_next_region =  hrom_out[4:0];
assign h_next_evnt =    hrom_out[15:5];
assign h_region_rom =   hrom_out[20:16];
assign h_clr =          hrom_out[21];
assign v_inc =          hrom_out[22];
assign trs =            hrom_out[23];
assign h =              hrom_out[24];
assign usropt_rgn =     hrom_out[25];
assign ceqpol_rgn =     hrom_out[26];
assign eav2_rgn =       hrom_out[27];
assign sav2_rgn =       hrom_out[28];

//
// Horizontal enable
//
// This signal is the clock enable signal to the HROM. It is asserted when the
// horizontal counter value equals the value of the h_next_evnt field from the
// HROM AND when the h_counter LSB is high.
//
assign hrom_en = ce & h_counter[0] & h_evnt_match | rst;

//
// Horizontal counter
//
// The horizontal counter increments every clock cycle unless the h_clr signal
// from the HROM is asserted, in which case, it resets to a value of 0.
//
always @ (posedge clk or posedge rst)
    if (rst)
        h_counter <= 12'd4095;
    else if (ce)
        begin
            if (h_clr & h_counter[0])
                h_counter <= 0;
            else
                h_counter <= h_counter + 1;
        end

assign h_counter_lsb = h_counter[0];

//
// Horizontal event comparator
//
// This signal is asserted when the h_next_evnt value from the HROM matches the
// 11 MSBs of the h_counter.
//
assign h_evnt_match = (h_next_evnt == h_counter[HCNT_MSB:1]);

//
// Horizontal region encoder
// 
// This encoder can change the horizontal region value coming out of the horz
// ROM before it is used to address the color ROM. The horizontal region is
// modified under 3 conditions. 
//
// First, if the horizontal region is the first (leftmost) colorbar, then the 
// two user_opt inputs are applied to modify the horizontal region to affect 
// the color generated by the color ROM. 
//
// Second, if the region is the first sample of first line of the first field 
// of an even frame and the cable equalization pattern is being drawn, then 
// the sample must be drawn differently in order to provide both DC levels of 
// the cable equalization pattern. The horizontal ROM will indicate this 
// region by generating a region code of HRGN_CEQ_POL_0. The encoder will 
// normally change this to HRGN_CEQ_POL_0, unless this is the first active 
// line of the first field of an even frame, in which case it will output 
// HRGN_CEQ_POL_1 to the color ROM.
//
// Third, the color ROM needs to know the sense of the field bit in order to
// properly generate the XYZ word of EAVs and SAVs. The v_band does not carry
// the field information into the color ROM, so the field bit from the vertical
// ROM is used to modify the horizontal region during the second half of the
// EAV and SAV regions to indicate the status of the field bit.
//
always @*
    if (usropt_rgn)
        case(user_opt)
            2'b01: h_region = HRGN_USROPT1;
            2'b10: h_region = HRGN_USROPT2;
            2'b11: h_region = HRGN_USROPT3;
            default: h_region = HRGN_BAR1;
        endcase
    
    else if (ceqpol_rgn)
        begin
            if (first_line & h_counter[1:0] == 2'b00)
                h_region = HRGN_CEQ_POL_1;
            else
                h_region = HRGN_CEQ_POL_0;
        end
    
    else if (eav2_rgn)
        h_region = f ? HRGN_EAV2_F1 : HRGN_EAV2_F0;
    
    else if (sav2_rgn)
        h_region = f ? HRGN_SAV2_F1 : HRGN_SAV2_F0;

    else
        h_region = h_region_rom;

//
// XYZ signal decoding
//
assign xyz = (eav2_rgn | sav2_rgn) & h_counter[0];

endmodule
