//------------------------------------------------------------------------------
// multigenHD_output.v
//
// HD digital video YCbCr 4:2:2 video pattern generator output section
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2004 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Mar 2, 2004
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Mar  2, 2004       XAPP682 version 1.0 release
//
// Other modules instanced in this design:
//
//          None.
//------------------------------------------------------------------------------
/*
This module contains the output section of the HD video pattern generator. A
block RAM is used to convert the vertical and horizontal coordinates of the
video pattern into Y and C output values.

A Y-Ramp generator is used to create the Y-Ramp pattern for RP 219.      
*/

module multigenHD_output (
    input   wire        clk,            // word-rate clock
    input   wire        rst,            // async reset
    input   wire        ce,             // clock enable
    input   wire [4:0]  h_region,       // horizontal region
    input   wire [2:0]  v_band,         // vertical band
    input   wire        h_counter_lsb,  // LSB of horizontal counter
    input   wire        y_ramp_inc_sel, // controls the Y-Ramp increment MUX
    output  wire [9:0]  y,              // luma output channel
    output  wire [9:0]  c               // chroma output channel
);

//-----------------------------------------------------------------------------
// Parameter definitions
//

//
// This group of parameters defines the bit widths of various fields in the
// module. Note that if the VID_WIDTH parameter is changed, the video component
// values for the various colors will need to be modified accordingly.
//
parameter VID_WIDTH     = 10;                   // Width of video components
parameter HRGN_WIDTH    = 5;                    // Width of h_region counter
parameter VBAND_WIDTH   = 3;                    // Width of vband code
parameter YRAMP_FRAC    = 7;                    // Number of fractional bits in Y Ramp reg
parameter YRAMP_WIDTH   = VID_WIDTH + YRAMP_FRAC;// Width of Y Ramp reg

parameter VID_MSB       = VID_WIDTH - 1;        // MS bit # of video data path
parameter HRGN_MSB      = HRGN_WIDTH - 1;       // MS bit # of h_region counter
parameter VBAND_MSB     = VBAND_WIDTH - 1;      // MS bit # of vband code
parameter YRAMP_MSB     = YRAMP_WIDTH - 1;      // MS bit # of y_ramp register


//
// Y-ramp parameters
//
// These constants control the Y-Ramp pattern generation.
//
parameter YRAMP_INIT    = 17'b0000111111_1100000;   // 63.75 is initial Y ramp starting value
parameter Y_INC_1920    = 8'b0_1011011;             // 0.7109375 is ramp increment for 1920 wide standards
parameter Y_INC_1280    = 8'b1_0001001;             // 1.0703125 is ramp increment for 1280 wide standards
parameter YRAMP_RND     = 17'b0000000000_1000000;   // 0.5 is the round up increment value for rounder


//-----------------------------------------------------------------------------
// Signal definitions
//

wire                    y_ramp_reload;  // reloads the y_ramp reg
wire                    y_ramp_en;      // enables output of the y_ramp reg
reg     [YRAMP_MSB:0]   y_ramp;         // Y ramp register
wire    [YRAMP_MSB:0]   y_ramp_round;   // Y ramp rounder
wire    [VID_MSB:0]     y_ramp_out;     // rounded output of Y ramp reg
wire    [YRAMP_MSB:0]   y_ramp_inc;     // output of Y ramp increment MUX
wire    [31:0]          crom_out;       // CROM output
wire    [VID_MSB:0]     y_rom;          // Y output of color ROM
wire    [VID_MSB:0]     c_rom;          // C output of color ROM
wire    [VID_MSB:0]     y_ramp_mux;     // output of Y ramp mux
wire    [31:0]          GND32 = 32'b0000_0000_0000_0000_0000_0000_0000_0000;
wire    [3:0]           GND4 =  4'b0000;

    
//----------------------------------------------------------------------------
// Color ROM
//

// Simulation initialization code CROM
// Created by multigenHD_romgen.v
// Video format mapping:
//   000 =  SMPTE 295M - 1080i  25Hz (1250 lines/frame)
//   001 =  SMPTE 274M - 1080sF 24Hz & 23.98Hz         
//   002 =  SMPTE 274M - 1080i  30Hz & 29.97 Hz        
//   003 =  SMPTE 274M - 1080i  25Hz                   
//   004 =  SMPTE 274M - 1080p  30Hz & 29.97Hz         
//   005 =  SMPTE 274M - 1080p  25Hz                   
//   006 =  SMPTE 274M - 1080p  24Hz & 23.98Hz         
//   007 =  SMPTE 296M - 720p   60Hz & 59.94Hz         
//synthesis translate_off
defparam
CROM.INIT = 36'h000800100,
CROM.SRVAL = 36'h000800100,
CROM.WRITE_MODE = "READ_FIRST",
CROM.INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INITP_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INITP_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INITP_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INITP_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
CROM.INIT_00 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_01 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_02 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_03 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_04 = 256'h0000000000FFCFFC008001000080010000B60B60000000000000000000FFCFFC,
CROM.INIT_05 = 256'h00EC0EC0000000000080010000800100008001000080010000AB0AB000000000,
CROM.INIT_06 = 256'h00F10F1000000000008001000080010000800100008001000080010000800100,
CROM.INIT_07 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_08 = 256'h0087CA88002C0A880087CA88002C0A8800800B4400800B440080067800800678,
CROM.INIT_09 = 256'h00CC43EC00C0C3EC0033C858003F48580033C858003F4858002C091400934914,
CROM.INIT_0A = 256'h00D40330006CC33000D40330006CC33000CC43EC00C0C3EC00CC43EC00C0C3EC,
CROM.INIT_0B = 256'h00800678008006780080067800800678007841BC00D401BC00D40330006CC330,
CROM.INIT_0C = 256'h0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC,
CROM.INIT_0D = 256'h00C70C7000000000008001000080010000800100008001000080080000000000,
CROM.INIT_0E = 256'h00DA0DA00000000000800B4400800B4400800B4400800B4400800B4400800B44,
CROM.INIT_0F = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_10 = 256'h00800B4400800B4400800B4400800B4400800B4400800B4400100BC80099CBC8,
CROM.INIT_11 = 256'h00800B4400800B4400800B4400800B4400800B4400800B4400800B4400800B44,
CROM.INIT_12 = 256'h00800B4400800B4400800B4400800B4400800B4400800B4400800B4400800B44,
CROM.INIT_13 = 256'h0075C1FC00F001FC0075C1FC00F001FC00800B4400800B4400800B4400800B44,
CROM.INIT_14 = 256'h0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC,
CROM.INIT_15 = 256'h00C70C7000000000008001000080010000800100008001000080080000000000,
CROM.INIT_16 = 256'h00DA0DA0000000000062C3D0009903D0009D43D4006703D400800EB000800EB0,
CROM.INIT_17 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_18 = 256'h00800EB200800EB200800EB200800EB20080010100800101008A4DB400100DB4,
CROM.INIT_19 = 256'h00800EB200800EB200800EB200800EB200800EB200800EB200800EB200800EB2,
CROM.INIT_1A = 256'h00800EB200800EB200800EB200800EB200800EB200800EB200800EB200800EB2,
CROM.INIT_1B = 256'h00F003E8006643E800F003E8006643E800800EB200800EB200800EB200800EB2,
CROM.INIT_1C = 256'h0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC,
CROM.INIT_1D = 256'h00C70C7000000000008001000080010000800100008001000080080000000000,
CROM.INIT_1E = 256'h00DA0DA0000000000097823500AE423500800101008001010080010100800101,
CROM.INIT_1F = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_20 = 256'h00800EB000800EB0008001000080010000800100008001000080030C0080030C,
CROM.INIT_21 = 256'h0080010000800100008001000080010000800EB000800EB000800EB000800EB0,
CROM.INIT_22 = 256'h008001000080010000800148008001480080010000800100008000B8008000B8,
CROM.INIT_23 = 256'h0080030C0080030C0080030C0080030C00800100008001000080018C0080018C,
CROM.INIT_24 = 256'h0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC,
CROM.INIT_25 = 256'h00C70C7000000000008001000080010000800100008001000080080000000000,
CROM.INIT_26 = 256'h00DA0DA000000000008001000080010000800100008001000080010000800100,
CROM.INIT_27 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_28 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_29 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_2A = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_2B = 256'h00C0066000C00660008001000080010000800100008001000080010000800100,
CROM.INIT_2C = 256'h0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC,
CROM.INIT_2D = 256'h00C70C7000000000008001000080010000800100008001000080080000000000,
CROM.INIT_2E = 256'h00DA0DA000000000008001000080010000800100008001000080010000800100,
CROM.INIT_2F = 256'h008001000080010000C0064000C0064000C0066000C0066000C0066000C00660,
CROM.INIT_30 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_31 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_32 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_33 = 256'h0080044000800440008001000080010000800100008001000080010000800100,
CROM.INIT_34 = 256'h0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC,
CROM.INIT_35 = 256'h00C70C7000000000008001000080010000800100008001000080080000000000,
CROM.INIT_36 = 256'h00DA0DA000000000008001000080010000800100008001000080010000800100,
CROM.INIT_37 = 256'h0080010000800100008004400080044000800440008004400080044000800440,
CROM.INIT_38 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_39 = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_3A = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_3B = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_3C = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_3D = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_3E = 256'h0080010000800100008001000080010000800100008001000080010000800100,
CROM.INIT_3F = 256'h0080010000800100008001000080010000800100008001000080010000800100;
//synthesis translate_on

RAMB16_S36 CROM (
    .DO     (crom_out),
    .DOP    (),
    .ADDR   ({v_band, h_region, h_counter_lsb}),
    .CLK    (clk),
    .DI     (GND32),
    .DIP    (GND4),
    .EN     (1'b1),
    .SSR    (rst),
    .WE     (1'b0));

// XST synthesis initialization code CROM
// Created by multigenHD_romgen.v
// Video format mapping:
//   000 =  SMPTE 295M - 1080i  25Hz (1250 lines/frame)
//   001 =  SMPTE 274M - 1080sF 24Hz & 23.98Hz         
//   002 =  SMPTE 274M - 1080i  30Hz & 29.97 Hz        
//   003 =  SMPTE 274M - 1080i  25Hz                   
//   004 =  SMPTE 274M - 1080p  30Hz & 29.97Hz         
//   005 =  SMPTE 274M - 1080p  25Hz                   
//   006 =  SMPTE 274M - 1080p  24Hz & 23.98Hz         
//   007 =  SMPTE 296M - 720p   60Hz & 59.94Hz         
//synthesis attribute INIT of CROM is "000800100"
//synthesis attribute SRVAL of CROM is "000800100"
//synthesis attribute WRITE_MODE of CROM is "READ_FIRST"
//synthesis attribute INITP_00 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_01 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_02 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_03 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_04 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_05 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_06 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_07 of CROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INIT_00 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_01 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_02 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_03 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_04 of CROM is "0000000000FFCFFC008001000080010000B60B60000000000000000000FFCFFC"
//synthesis attribute INIT_05 of CROM is "00EC0EC0000000000080010000800100008001000080010000AB0AB000000000"
//synthesis attribute INIT_06 of CROM is "00F10F1000000000008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_07 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_08 of CROM is "0087CA88002C0A880087CA88002C0A8800800B4400800B440080067800800678"
//synthesis attribute INIT_09 of CROM is "00CC43EC00C0C3EC0033C858003F48580033C858003F4858002C091400934914"
//synthesis attribute INIT_0A of CROM is "00D40330006CC33000D40330006CC33000CC43EC00C0C3EC00CC43EC00C0C3EC"
//synthesis attribute INIT_0B of CROM is "00800678008006780080067800800678007841BC00D401BC00D40330006CC330"
//synthesis attribute INIT_0C of CROM is "0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC"
//synthesis attribute INIT_0D of CROM is "00C70C7000000000008001000080010000800100008001000080080000000000"
//synthesis attribute INIT_0E of CROM is "00DA0DA00000000000800B4400800B4400800B4400800B4400800B4400800B44"
//synthesis attribute INIT_0F of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_10 of CROM is "00800B4400800B4400800B4400800B4400800B4400800B4400100BC80099CBC8"
//synthesis attribute INIT_11 of CROM is "00800B4400800B4400800B4400800B4400800B4400800B4400800B4400800B44"
//synthesis attribute INIT_12 of CROM is "00800B4400800B4400800B4400800B4400800B4400800B4400800B4400800B44"
//synthesis attribute INIT_13 of CROM is "0075C1FC00F001FC0075C1FC00F001FC00800B4400800B4400800B4400800B44"
//synthesis attribute INIT_14 of CROM is "0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC"
//synthesis attribute INIT_15 of CROM is "00C70C7000000000008001000080010000800100008001000080080000000000"
//synthesis attribute INIT_16 of CROM is "00DA0DA0000000000062C3D0009903D0009D43D4006703D400800EB000800EB0"
//synthesis attribute INIT_17 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_18 of CROM is "00800EB200800EB200800EB200800EB20080010100800101008A4DB400100DB4"
//synthesis attribute INIT_19 of CROM is "00800EB200800EB200800EB200800EB200800EB200800EB200800EB200800EB2"
//synthesis attribute INIT_1A of CROM is "00800EB200800EB200800EB200800EB200800EB200800EB200800EB200800EB2"
//synthesis attribute INIT_1B of CROM is "00F003E8006643E800F003E8006643E800800EB200800EB200800EB200800EB2"
//synthesis attribute INIT_1C of CROM is "0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC"
//synthesis attribute INIT_1D of CROM is "00C70C7000000000008001000080010000800100008001000080080000000000"
//synthesis attribute INIT_1E of CROM is "00DA0DA0000000000097823500AE423500800101008001010080010100800101"
//synthesis attribute INIT_1F of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_20 of CROM is "00800EB000800EB0008001000080010000800100008001000080030C0080030C"
//synthesis attribute INIT_21 of CROM is "0080010000800100008001000080010000800EB000800EB000800EB000800EB0"
//synthesis attribute INIT_22 of CROM is "008001000080010000800148008001480080010000800100008000B8008000B8"
//synthesis attribute INIT_23 of CROM is "0080030C0080030C0080030C0080030C00800100008001000080018C0080018C"
//synthesis attribute INIT_24 of CROM is "0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC"
//synthesis attribute INIT_25 of CROM is "00C70C7000000000008001000080010000800100008001000080080000000000"
//synthesis attribute INIT_26 of CROM is "00DA0DA000000000008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_27 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_28 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_29 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_2A of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_2B of CROM is "00C0066000C00660008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_2C of CROM is "0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC"
//synthesis attribute INIT_2D of CROM is "00C70C7000000000008001000080010000800100008001000080080000000000"
//synthesis attribute INIT_2E of CROM is "00DA0DA000000000008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_2F of CROM is "008001000080010000C0064000C0064000C0066000C0066000C0066000C00660"
//synthesis attribute INIT_30 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_31 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_32 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_33 of CROM is "0080044000800440008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_34 of CROM is "0000000000FFCFFC0080010000800100009D09D0000000000000000000FFCFFC"
//synthesis attribute INIT_35 of CROM is "00C70C7000000000008001000080010000800100008001000080080000000000"
//synthesis attribute INIT_36 of CROM is "00DA0DA000000000008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_37 of CROM is "0080010000800100008004400080044000800440008004400080044000800440"
//synthesis attribute INIT_38 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_39 of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_3A of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_3B of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_3C of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_3D of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_3E of CROM is "0080010000800100008001000080010000800100008001000080010000800100"
//synthesis attribute INIT_3F of CROM is "0080010000800100008001000080010000800100008001000080010000800100"

assign y_ramp_reload = crom_out[0];
assign y_ramp_en = crom_out[1];
assign y_rom = crom_out[VID_MSB+2:2];
assign c_rom = crom_out[VID_MSB+14:14];

//
// Y Ramp increment selection
//
// This MUX selects the Y Ramp increment value. Different increment values are
// used for formats with 1920 active samples per line vs. 1280 active samples
// per line. This is because the Y Ramp pattern contains less samples if there
// are only 1280 active samples per line, so the increment value must be
// bigger in order to reach the maximum Y value by the end of the Y Ramp
// pattern.
//
// The control for this MUX comes from an output of the VROM. The VROM decodes
// the std input code and controls this MUX appropriately.
//
assign y_ramp_inc = y_ramp_inc_sel ? Y_INC_1280 : Y_INC_1920;

//
// Y Ramp register & adder
//
// This is the accumulator section of the Y Ramp generator. The register
// normally accumulates new Y Ramp increment values with the current contents
// of the register every clock cycle. However, if the y_ramp_reload output of
// the CROM is asserted, the register is loaded with the YRAMP_INIT value to
// begin the Y_RAMP pattern. The accumulator contains 10 integer bits and 7
// fractional bits to make a smooth ramp function.
//
always @ (posedge clk or posedge rst)
    if (rst)
        y_ramp <= YRAMP_INIT;
    else if (ce)
        begin
            if (y_ramp_reload)
                y_ramp <= YRAMP_INIT;
            else
                y_ramp <= y_ramp + y_ramp_inc;
        end

//
// Y Ramp rounder
//
// This code rounds the output of the Y Ramp accumulator to the nearest integer
// by adding a value of 0.5 and then truncating the fractional bits.
//
assign y_ramp_round = y_ramp + YRAMP_RND;
assign y_ramp_out = y_ramp_round[YRAMP_MSB:(YRAMP_MSB - VID_WIDTH) + 1];

//
// Y output mux
//
// This MUX will normally output the Y value from the CROM unless the Y Ramp
// pattern is being shown, in which case it outputs the rounded output of the
// Y Ramp accumulator.
//
assign y_ramp_mux = y_ramp_en ? y_ramp_out : y_rom;

assign y = y_ramp_mux;
assign c = c_rom;

endmodule
