//------------------------------------------------------------------------------
// multigenHD_vert.v
//
// HD digital video YCbCr 4:2:2 video pattern generator vertical section
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2004 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Mar 2, 2004
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Mar  2, 2004       XAPP682 version 1.0 release
//
// Other modules instanced in this design:
//
//          None.
//------------------------------------------------------------------------------
/*
This file contains the vertical sequencer for the HD video pattern generator.
A block RAM is used as a finite state machine, sequencing through the various
vertical sections of each video pattern. The module outputs a v_band code 
indicating which vertical portion of the video pattern should be displayed.      
*/

module multigenHD_vert (
    input   wire        clk,            // word-rate clock
    input   wire        rst,            // async reset
    input   wire        ce,             // clock enable
    input   wire [2:0]  std,            // selects video format
    input   wire [1:0]  pattern,        // 00 = RP 219 colorbars, X1 = RP 198 checkfield, 10 = 75% colorbars
    input   wire        h_counter_lsb,  // LSB of the horz section's h_counter
    input   wire        v_inc,          // causes the vertical counter to increment
    output  wire [2:0]  v_band,         // vertical band code output
    output  wire        v,              // vertical blanking interval indicator
    output  wire        f,              // odd/even field bit
    output  wire        first_line,     // asserted during first active line
    output  wire        y_ramp_inc_sel, // controls output sections Y-Ramp increment MUX
    output  wire [10:0] line_num        // current vertical line number
);

//-----------------------------------------------------------------------------
// Parameter definitions
//

//
// This group of parameters defines the bit widths of various fields in the
// module. 
//
parameter VID_WIDTH     = 10;                   // Width of video components
parameter VCNT_WIDTH    = 11;                   // Width of v_counter
parameter VRGN_WIDTH    = 5;                    // Width of v_region counter
parameter VBAND_WIDTH   = 3;                    // Width of vband code

parameter V_EVNT_WIDTH  = VCNT_WIDTH;           // Width of v_next_evnt
 
parameter VID_MSB       = VID_WIDTH - 1;        // MS bit # of video data path
parameter VCNT_MSB      = VCNT_WIDTH - 1;       // MS bit # of v_counter
parameter VRGN_MSB      = VRGN_WIDTH - 1;       // MS bit # of v_region counter
parameter VBAND_MSB     = VBAND_WIDTH - 1;      // MS bit # of vband code
parameter V_EVNT_MSB    = V_EVNT_WIDTH - 1;     // MS bit # of v_next_evnt

//
// The group of parameters defines the vertical regions from the vertical ROM.
// Note that the vertical ROM generates a 5-bit vertical region code and a 3-bit
// vertical band code. The region code is essentially the current state of the
// vertical state machine and feeds back to the input of the vertical ROM.
// The vertical band code is sent to the color ROM to indicate the current
// vertical pattern band.
//
parameter [VRGN_MSB:0]
    VRGN_FM0_F0_VB_0    = 0,    // frame 0, field 0, first vertical blanking interval
    VRGN_FM0_F0_1ST_ACT = 1,    // frame 0, field 0, first active line (for CEQ polarity)
    VRGN_FM0_F0_PAT1    = 2,    // frame 0, field 0, pattern 1 or cable eq pattern
    VRGN_FM0_F0_PAT2    = 3,    // frame 0, field 0, pattern 2 or PLL pattern
    VRGN_FM0_F0_PAT3    = 4,    // frame 0, field 0, pattern 3
    VRGN_FM0_F0_PAT4    = 5,    // frame 0, field 0, pattern 4
    VRGN_FM0_F0_VB_1    = 6,    // frame 0, field 0, second vertical blanking interval
    VRGN_FM0_F1_VB_0    = 7,    // frame 0, field 1, first vertical blanking interval
    VRGN_FM0_F1_PAT1    = 8,    // frame 0, field 1, pattern 1 or cable eq pattern
    VRGN_FM0_F1_PAT2    = 9,    // frame 0, field 1, pattern 2 or PLL pattern
    VRGN_FM0_F1_PAT3    = 10,   // frame 0, field 1, pattern 3
    VRGN_FM0_F1_PAT4    = 11,   // frame 0, field 1, pattern 4
    VRGN_FM0_F1_VB_1    = 12,   // frame 0, field 1, second vertical blanking interval
    VRGN_FM0_CLRV       = 13,   // frame 0 clears the vertical counter back to 1
    VRGN_X14            = 14,   // unused
    VRGN_X15            = 15,   // unused
    VRGN_FM1_F0_VB_0    = 16,   // frame 1, field 0, first vertical blanking interval
    VRGN_FM1_F0_PAT1    = 17,   // frame 1, field 0, pattern 1 or cable eq pattern
    VRGN_FM1_F0_PAT2    = 18,   // frame 1, field 0, pattern 2 or PLL pattern
    VRGN_FM1_F0_PAT3    = 19,   // frame 1, field 0, pattern 3
    VRGN_FM1_F0_PAT4    = 20,   // frame 1, field 0, pattern 4
    VRGN_FM1_F0_VB_1    = 21,   // frame 1, field 0, second vertical blanking interval
    VRGN_X22            = 22,   // unused
    VRGN_FM1_F1_VB_0    = 23,   // frame 1, field 1, first vertical blanking interval
    VRGN_FM1_F1_PAT1    = 24,   // frame 1, field 1, pattern 1 or cable eq pattern
    VRGN_FM1_F1_PAT2    = 25,   // frame 1, field 1, pattern 2 or PLL pattern
    VRGN_FM1_F1_PAT3    = 26,   // frame 1, field 1, pattern 3
    VRGN_FM1_F1_PAT4    = 27,   // frame 1, field 1, pattern 4
    VRGN_FM1_F1_VB_1    = 28,   // frame 1, field 1, second vertical blanking interval
    VRGN_FM1_CLRV       = 29,   // frame 1 clears the vertical counter back to 1
    VRGN_X30            = 30,   // unused
    VRGN_RST            = 31;   // initial state after reset

parameter [VBAND_MSB:0]
    VBAND_VB            = 0,    // vertical blanking interval
    VBAND_PAT1          = 1,    // pattern 1
    VBAND_PAT2          = 2,    // pattern 2
    VBAND_PAT3          = 3,    // pattern 3
    VBAND_PAT4          = 4,    // pattern 4
    VBAND_CEQ           = 5,    // cable equalization pattern
    VBAND_PLL           = 6,    // PLL pattern
    VBAND_X7            = 7;    // unused

//-----------------------------------------------------------------------------
// Signal definitions
//

wire    [31:0]          vrom_out;       // VROM output
reg     [VCNT_MSB:0]    v_counter;      // vertical counter
wire    [V_EVNT_MSB:0]  v_next_evnt;    // next vertical event
wire                    v_evnt_match;   // output of vertical event comparator
wire                    vrom_en;        // EN input to vertical ROM
wire    [VRGN_MSB:0]    v_region;       // current vertical region
wire    [VBAND_MSB:0]   v_band_rom;     // VBAND for most patterns
wire    [VBAND_MSB:0]   v_band_75_rom;  // VBAND for 75% color bars pattern
wire                    v_clr;          // vertical counter clear signal
wire    [31:0]          GND32 = 32'b0000_0000_0000_0000_0000_0000_0000_0000;
wire    [3:0]           GND4 =  4'b0000;

//
// Vertical ROM
//

// Simulation initialization code VROM
// Created by multigenHD_romgen.v
// Video format mapping:
//   000 =  SMPTE 295M - 1080i  25Hz (1250 lines/frame)
//   001 =  SMPTE 274M - 1080sF 24Hz & 23.98Hz         
//   002 =  SMPTE 274M - 1080i  30Hz & 29.97 Hz        
//   003 =  SMPTE 274M - 1080i  25Hz                   
//   004 =  SMPTE 274M - 1080p  30Hz & 29.97Hz         
//   005 =  SMPTE 274M - 1080p  25Hz                   
//   006 =  SMPTE 274M - 1080p  24Hz & 23.98Hz         
//   007 =  SMPTE 296M - 720p   60Hz & 59.94Hz         
//synthesis translate_off
defparam
VROM.INIT = 36'h00048FFFF,
VROM.SRVAL = 36'h00048FFFF,
VROM.WRITE_MODE = "READ_FIRST",
VROM.INITP_00 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INITP_01 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INITP_02 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INITP_03 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INITP_04 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INITP_05 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INITP_06 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INITP_07 = 256'h0000000000000000000000000000000000000000000000000000000000000000,
VROM.INIT_00 = 256'h0018582800084E2702044D8602033CA5020237040201316302810A2200080A01,
VROM.INIT_01 = 256'h00089C4000089C4000589C5000189C2D02149BAC02138ACB0212852A02117F89,
VROM.INIT_02 = 256'h0038583800089C4000284E3702244D9502233CB4022237130221317200280A11,
VROM.INIT_03 = 256'h0008002000089C4000789C4000389C3D02349BBC02338ADB0232853A02317F99,
VROM.INIT_04 = 256'h0018582800084E2702040006020300050C064D860A052BC30A850A2200080A01,
VROM.INIT_05 = 256'h00089C4000089C4000589C5000189C2D0214000C0213000B0C169BAC0A1579E9,
VROM.INIT_06 = 256'h0038583800089C4000284E3702240015022300140C264D950A252BD200280A11,
VROM.INIT_07 = 256'h0008002000089C4000789C4000389C3D0234001C0233001B0C369BBC0A3579F9,
VROM.INIT_08 = 256'h001848E800084667020446060203352502022F84020129E3028102A200080281,
VROM.INIT_09 = 256'h00088CA000088CA000588CB000188C8D02148C6C02137B8B021275EA02117049,
VROM.INIT_0A = 256'h003848F800088CA000284677022446150223353402222F93022129F200280291,
VROM.INIT_0B = 256'h0008002000088CA000788CA000388C9D02348C7C02337B9B023275FA02317059,
VROM.INIT_0C = 256'h001848E80008466702040006020300050C0646060A0524430A8502A200080281,
VROM.INIT_0D = 256'h00088CA000088CA000588CB000188C8D0214000C0213000B0C168C6C0A156AA9,
VROM.INIT_0E = 256'h003848F800088CA00028467702240015022300140C2646150A25245200280291,
VROM.INIT_0F = 256'h0008002000088CA000788CA000388C9D0234001C0233001B0C368C7C0A356AB9,
VROM.INIT_10 = 256'h001848E800084667020446060203352502022F84020129E3028102A200080281,
VROM.INIT_11 = 256'h00088CA000088CA000588CB000188C8D02148C6C02137B8B021275EA02117049,
VROM.INIT_12 = 256'h003848F800088CA000284677022446150223353402222F93022129F200280291,
VROM.INIT_13 = 256'h0008002000088CA000788CA000388C9D02348C7C02337B9B023275FA02317059,
VROM.INIT_14 = 256'h001848E80008466702040006020300050C0646060A0524430A8502A200080281,
VROM.INIT_15 = 256'h00088CA000088CA000588CB000188C8D0214000C0213000B0C168C6C0A156AA9,
VROM.INIT_16 = 256'h003848F800088CA00028467702240015022300140C2646150A25245200280291,
VROM.INIT_17 = 256'h0008002000088CA000788CA000388C9D0234001C0233001B0C368C7C0A356AB9,
VROM.INIT_18 = 256'h001848E800084667020446060203352502022F84020129E3028102A200080281,
VROM.INIT_19 = 256'h00088CA000088CA000588CB000188C8D02148C6C02137B8B021275EA02117049,
VROM.INIT_1A = 256'h003848F800088CA000284677022446150223353402222F93022129F200280291,
VROM.INIT_1B = 256'h0008002000088CA000788CA000388C9D02348C7C02337B9B023275FA02317059,
VROM.INIT_1C = 256'h001848E80008466702040006020300050C0646060A0524430A8502A200080281,
VROM.INIT_1D = 256'h00088CA000088CA000588CB000188C8D0214000C0213000B0C168C6C0A156AA9,
VROM.INIT_1E = 256'h003848F800088CA00028467702240015022300140C2646150A25245200280291,
VROM.INIT_1F = 256'h0008002000088CA000788CA000388C9D0234001C0233001B0C368C7C0A356AB9,
VROM.INIT_20 = 256'h0018000800088C8D02048C2602036A6502025F24020153E30281054200080521,
VROM.INIT_21 = 256'h00088CA000088CA000488CB000188C8D0214000C0213000B0212000A02110009,
VROM.INIT_22 = 256'h0038001800088CA000288C9D02248C3502236A7402225F33022153F200280531,
VROM.INIT_23 = 256'h0008002000088CA000688CA000388C9D0234001C0233001B0232001A02310019,
VROM.INIT_24 = 256'h0018000800088C8D02040006020300050C068C260A0548A30A85054200080521,
VROM.INIT_25 = 256'h00088CA000088CA000488CB000188C8D0214000C0213000B0C16000C0A150009,
VROM.INIT_26 = 256'h0038001800088CA000288C9D02240015022300140C268C350A2548B200280531,
VROM.INIT_27 = 256'h0008002000088CA000688CA000388C9D0234001C0233001B0C36001C0A350019,
VROM.INIT_28 = 256'h0018000800088C8D02048C2602036A6502025F24020153E30281054200080521,
VROM.INIT_29 = 256'h00088CA000088CA000488CB000188C8D0214000C0213000B0212000A02110009,
VROM.INIT_2A = 256'h0038001800088CA000288C9D02248C3502236A7402225F33022153F200280531,
VROM.INIT_2B = 256'h0008002000088CA000688CA000388C9D0234001C0233001B0232001A02310019,
VROM.INIT_2C = 256'h0018000800088C8D02040006020300050C068C260A0548A30A85054200080521,
VROM.INIT_2D = 256'h00088CA000088CA000488CB000188C8D0214000C0213000B0C16000C0A150009,
VROM.INIT_2E = 256'h0038001800088CA000288C9D02240015022300140C268C350A2548B200280531,
VROM.INIT_2F = 256'h0008002000088CA000688CA000388C9D0234001C0233001B0C36001C0A350019,
VROM.INIT_30 = 256'h0018000800088C8D02048C2602036A6502025F24020153E30281054200080521,
VROM.INIT_31 = 256'h00088CA000088CA000488CB000188C8D0214000C0213000B0212000A02110009,
VROM.INIT_32 = 256'h0038001800088CA000288C9D02248C3502236A7402225F33022153F200280531,
VROM.INIT_33 = 256'h0008002000088CA000688CA000388C9D0234001C0233001B0232001A02310019,
VROM.INIT_34 = 256'h0018000800088C8D02040006020300050C068C260A0548A30A85054200080521,
VROM.INIT_35 = 256'h00088CA000088CA000488CB000188C8D0214000C0213000B0C16000C0A150009,
VROM.INIT_36 = 256'h0038001800088CA000288C9D02240015022300140C268C350A2548B200280531,
VROM.INIT_37 = 256'h0008002000088CA000688CA000388C9D0234001C0233001B0C36001C0A350019,
VROM.INIT_38 = 256'h0118000801085DAD03045D26030346A503023F24030137A30381034201080321,
VROM.INIT_39 = 256'h01085DC001085DC001485DD001185DAD0314000C0313000B0312000A03110009,
VROM.INIT_3A = 256'h0138001801085DC001285DBD03245D35032346B403223F33032137B201280331,
VROM.INIT_3B = 256'h0108002001085DC001685DC001385DBD0334001C0333001B0332001A03310019,
VROM.INIT_3C = 256'h0118000801085DAD03040006030300050D065D260B0530230B85034201080321,
VROM.INIT_3D = 256'h01085DC001085DC001485DD001185DAD0314000C0313000B0D16000C0B150009,
VROM.INIT_3E = 256'h0138001801085DC001285DBD03240015032300140D265D350B25303201280331,
VROM.INIT_3F = 256'h0108002001085DC001685DC001385DBD0334001C0333001B0D36001C0B350019;
//synthesis translate_on

RAMB16_S36 VROM (
    .DO     (vrom_out),
    .DOP    (),
    .ADDR   ({std, pattern[0], v_region}),
    .CLK    (clk),
    .DI     (GND32),
    .DIP    (GND4),
    .EN     (vrom_en),
    .SSR    (rst),
    .WE     (1'b0));

// XST synthesis initialization code VROM
// Created by multigenHD_romgen.v
// Video format mapping:
//   000 =  SMPTE 295M - 1080i  25Hz (1250 lines/frame)
//   001 =  SMPTE 274M - 1080sF 24Hz & 23.98Hz         
//   002 =  SMPTE 274M - 1080i  30Hz & 29.97 Hz        
//   003 =  SMPTE 274M - 1080i  25Hz                   
//   004 =  SMPTE 274M - 1080p  30Hz & 29.97Hz         
//   005 =  SMPTE 274M - 1080p  25Hz                   
//   006 =  SMPTE 274M - 1080p  24Hz & 23.98Hz         
//   007 =  SMPTE 296M - 720p   60Hz & 59.94Hz         
//synthesis attribute INIT of VROM is "00048FFFF"
//synthesis attribute SRVAL of VROM is "00048FFFF"
//synthesis attribute WRITE_MODE of VROM is "READ_FIRST"
//synthesis attribute INITP_00 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_01 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_02 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_03 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_04 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_05 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_06 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INITP_07 of VROM is "0000000000000000000000000000000000000000000000000000000000000000"
//synthesis attribute INIT_00 of VROM is "0018582800084E2702044D8602033CA5020237040201316302810A2200080A01"
//synthesis attribute INIT_01 of VROM is "00089C4000089C4000589C5000189C2D02149BAC02138ACB0212852A02117F89"
//synthesis attribute INIT_02 of VROM is "0038583800089C4000284E3702244D9502233CB4022237130221317200280A11"
//synthesis attribute INIT_03 of VROM is "0008002000089C4000789C4000389C3D02349BBC02338ADB0232853A02317F99"
//synthesis attribute INIT_04 of VROM is "0018582800084E2702040006020300050C064D860A052BC30A850A2200080A01"
//synthesis attribute INIT_05 of VROM is "00089C4000089C4000589C5000189C2D0214000C0213000B0C169BAC0A1579E9"
//synthesis attribute INIT_06 of VROM is "0038583800089C4000284E3702240015022300140C264D950A252BD200280A11"
//synthesis attribute INIT_07 of VROM is "0008002000089C4000789C4000389C3D0234001C0233001B0C369BBC0A3579F9"
//synthesis attribute INIT_08 of VROM is "001848E800084667020446060203352502022F84020129E3028102A200080281"
//synthesis attribute INIT_09 of VROM is "00088CA000088CA000588CB000188C8D02148C6C02137B8B021275EA02117049"
//synthesis attribute INIT_0A of VROM is "003848F800088CA000284677022446150223353402222F93022129F200280291"
//synthesis attribute INIT_0B of VROM is "0008002000088CA000788CA000388C9D02348C7C02337B9B023275FA02317059"
//synthesis attribute INIT_0C of VROM is "001848E80008466702040006020300050C0646060A0524430A8502A200080281"
//synthesis attribute INIT_0D of VROM is "00088CA000088CA000588CB000188C8D0214000C0213000B0C168C6C0A156AA9"
//synthesis attribute INIT_0E of VROM is "003848F800088CA00028467702240015022300140C2646150A25245200280291"
//synthesis attribute INIT_0F of VROM is "0008002000088CA000788CA000388C9D0234001C0233001B0C368C7C0A356AB9"
//synthesis attribute INIT_10 of VROM is "001848E800084667020446060203352502022F84020129E3028102A200080281"
//synthesis attribute INIT_11 of VROM is "00088CA000088CA000588CB000188C8D02148C6C02137B8B021275EA02117049"
//synthesis attribute INIT_12 of VROM is "003848F800088CA000284677022446150223353402222F93022129F200280291"
//synthesis attribute INIT_13 of VROM is "0008002000088CA000788CA000388C9D02348C7C02337B9B023275FA02317059"
//synthesis attribute INIT_14 of VROM is "001848E80008466702040006020300050C0646060A0524430A8502A200080281"
//synthesis attribute INIT_15 of VROM is "00088CA000088CA000588CB000188C8D0214000C0213000B0C168C6C0A156AA9"
//synthesis attribute INIT_16 of VROM is "003848F800088CA00028467702240015022300140C2646150A25245200280291"
//synthesis attribute INIT_17 of VROM is "0008002000088CA000788CA000388C9D0234001C0233001B0C368C7C0A356AB9"
//synthesis attribute INIT_18 of VROM is "001848E800084667020446060203352502022F84020129E3028102A200080281"
//synthesis attribute INIT_19 of VROM is "00088CA000088CA000588CB000188C8D02148C6C02137B8B021275EA02117049"
//synthesis attribute INIT_1A of VROM is "003848F800088CA000284677022446150223353402222F93022129F200280291"
//synthesis attribute INIT_1B of VROM is "0008002000088CA000788CA000388C9D02348C7C02337B9B023275FA02317059"
//synthesis attribute INIT_1C of VROM is "001848E80008466702040006020300050C0646060A0524430A8502A200080281"
//synthesis attribute INIT_1D of VROM is "00088CA000088CA000588CB000188C8D0214000C0213000B0C168C6C0A156AA9"
//synthesis attribute INIT_1E of VROM is "003848F800088CA00028467702240015022300140C2646150A25245200280291"
//synthesis attribute INIT_1F of VROM is "0008002000088CA000788CA000388C9D0234001C0233001B0C368C7C0A356AB9"
//synthesis attribute INIT_20 of VROM is "0018000800088C8D02048C2602036A6502025F24020153E30281054200080521"
//synthesis attribute INIT_21 of VROM is "00088CA000088CA000488CB000188C8D0214000C0213000B0212000A02110009"
//synthesis attribute INIT_22 of VROM is "0038001800088CA000288C9D02248C3502236A7402225F33022153F200280531"
//synthesis attribute INIT_23 of VROM is "0008002000088CA000688CA000388C9D0234001C0233001B0232001A02310019"
//synthesis attribute INIT_24 of VROM is "0018000800088C8D02040006020300050C068C260A0548A30A85054200080521"
//synthesis attribute INIT_25 of VROM is "00088CA000088CA000488CB000188C8D0214000C0213000B0C16000C0A150009"
//synthesis attribute INIT_26 of VROM is "0038001800088CA000288C9D02240015022300140C268C350A2548B200280531"
//synthesis attribute INIT_27 of VROM is "0008002000088CA000688CA000388C9D0234001C0233001B0C36001C0A350019"
//synthesis attribute INIT_28 of VROM is "0018000800088C8D02048C2602036A6502025F24020153E30281054200080521"
//synthesis attribute INIT_29 of VROM is "00088CA000088CA000488CB000188C8D0214000C0213000B0212000A02110009"
//synthesis attribute INIT_2A of VROM is "0038001800088CA000288C9D02248C3502236A7402225F33022153F200280531"
//synthesis attribute INIT_2B of VROM is "0008002000088CA000688CA000388C9D0234001C0233001B0232001A02310019"
//synthesis attribute INIT_2C of VROM is "0018000800088C8D02040006020300050C068C260A0548A30A85054200080521"
//synthesis attribute INIT_2D of VROM is "00088CA000088CA000488CB000188C8D0214000C0213000B0C16000C0A150009"
//synthesis attribute INIT_2E of VROM is "0038001800088CA000288C9D02240015022300140C268C350A2548B200280531"
//synthesis attribute INIT_2F of VROM is "0008002000088CA000688CA000388C9D0234001C0233001B0C36001C0A350019"
//synthesis attribute INIT_30 of VROM is "0018000800088C8D02048C2602036A6502025F24020153E30281054200080521"
//synthesis attribute INIT_31 of VROM is "00088CA000088CA000488CB000188C8D0214000C0213000B0212000A02110009"
//synthesis attribute INIT_32 of VROM is "0038001800088CA000288C9D02248C3502236A7402225F33022153F200280531"
//synthesis attribute INIT_33 of VROM is "0008002000088CA000688CA000388C9D0234001C0233001B0232001A02310019"
//synthesis attribute INIT_34 of VROM is "0018000800088C8D02040006020300050C068C260A0548A30A85054200080521"
//synthesis attribute INIT_35 of VROM is "00088CA000088CA000488CB000188C8D0214000C0213000B0C16000C0A150009"
//synthesis attribute INIT_36 of VROM is "0038001800088CA000288C9D02240015022300140C268C350A2548B200280531"
//synthesis attribute INIT_37 of VROM is "0008002000088CA000688CA000388C9D0234001C0233001B0C36001C0A350019"
//synthesis attribute INIT_38 of VROM is "0118000801085DAD03045D26030346A503023F24030137A30381034201080321"
//synthesis attribute INIT_39 of VROM is "01085DC001085DC001485DD001185DAD0314000C0313000B0312000A03110009"
//synthesis attribute INIT_3A of VROM is "0138001801085DC001285DBD03245D35032346B403223F33032137B201280331"
//synthesis attribute INIT_3B of VROM is "0108002001085DC001685DC001385DBD0334001C0333001B0332001A03310019"
//synthesis attribute INIT_3C of VROM is "0118000801085DAD03040006030300050D065D260B0530230B85034201080321"
//synthesis attribute INIT_3D of VROM is "01085DC001085DC001485DD001185DAD0314000C0313000B0D16000C0B150009"
//synthesis attribute INIT_3E of VROM is "0138001801085DC001285DBD03240015032300140D265D350B25303201280331"
//synthesis attribute INIT_3F of VROM is "0108002001085DC001685DC001385DBD0334001C0333001B0D36001C0B350019"

assign v_region =      vrom_out[4:0];
assign v_next_evnt =   vrom_out[15:5];
assign v_band_rom =    vrom_out[18:16];
assign v =             vrom_out[19];
assign f =             vrom_out[20];
assign v_clr =         vrom_out[22];
assign first_line =    vrom_out[23];
assign y_ramp_inc_sel =vrom_out[24];
assign v_band_75_rom = vrom_out[27:25];
//
// vrom_en
//
// This signal is asserted to advance the vertical sequencer. It is asserted
// whenever the vertical counter matches the next vertical event value AND
// the v_inc signal from the HROM is asserted AND the LSB of the h_counter is
// high.
//
assign vrom_en = ce & v_inc & h_counter_lsb & v_evnt_match | rst;

// 
// Vertical counter
//
// The vertical counter increments once per line. When the v_clr signal is
// asserted the counter resets to a value of 1.
//
always @ (posedge clk or posedge rst)
    if (rst)
        v_counter <= 11'd2047;
    else if (ce & h_counter_lsb)
        if (v_inc)
            begin
                if (v_clr)
                    v_counter <= 1;
                else 
                    v_counter <= v_counter + 1;
            end

assign line_num = v_counter;

//
// Vertical event comparator
//
// This compares the current vertical counter value with the v_next_evnt
// field from the VROM. When they match, v_evnt_match is asserted to enable
// clocking of the VROM.
//
assign v_evnt_match = (v_next_evnt == v_counter) | v_clr;

//
// v_band MUX
//
// When 75% color bars are being generated use the v_band_75_rom bits
// for the v_band, otherwise use v_band_rom.
//
assign v_band = pattern[1] ? v_band_75_rom : v_band_rom;


endmodule
